@@ -12,7 +12,7 @@ import re
 import shutil
 import subprocess
 from tempfile import gettempdir
-from typing import Any, Union, Optional
+from typing import Any, Optional, Union
 
 import click
 import tqdm
@@ -23,7 +23,7 @@ from pathvalidate import sanitize_filepath
 
 from . import converter
 from .clients import Client
-from .constants import FLAC_MAX_BLOCKSIZE, TRACK_FORMAT
+from .constants import FLAC_MAX_BLOCKSIZE, FOLDER_FORMAT, TRACK_FORMAT
 from .exceptions import (
     InvalidQuality,
     InvalidSourceError,
@@ -45,9 +45,7 @@ logger = logging.getLogger("streamrip")
 
 TYPE_REGEXES = {
     "remaster": re.compile(r"(?i)(re)?master(ed)?"),
-    "extra": re.compile(
-        r"(?i)(anniversary|deluxe|live|collector|demo|expanded)"
-    ),
+    "extra": re.compile(r"(?i)(anniversary|deluxe|live|collector|demo|expanded)"),
 }
 
 
@@ -90,6 +88,7 @@ class Track:
         self.downloaded = False
         self.tagged = False
         self.converted = False
+        self.part_of_tracklist = kwargs.get("part_of_tracklist", False)
 
         self.final_path: str
         self.container: str
@@ -102,7 +101,7 @@ class Track:
             self.meta = kwargs["meta"]
 
         if (u := kwargs.get("cover_url")) is not None:
-            logger.debug(f"Cover url: {u}")
+            logger.debug("Cover url: %s", u)
             self.cover_url = u
 
     def load_meta(self):
@@ -120,15 +119,12 @@ class Track:
             if self.client.source == "qobuz":
                 self.cover_url = self.resp["album"]["image"]["large"]
             elif self.client.source == "tidal":
-                self.cover_url = tidal_cover_url(
-                    self.resp["album"]["cover"], 320
-                )
+                self.cover_url = tidal_cover_url(self.resp["album"]["cover"], 320)
             elif self.client.source == "deezer":
                 self.cover_url = self.resp["album"]["cover_medium"]
             elif self.client.source == "soundcloud":
                 self.cover_url = (
-                    self.resp["artwork_url"]
-                    or self.resp["user"].get("avatar_url")
+                    self.resp["artwork_url"] or self.resp["user"].get("avatar_url")
                 ).replace("large", "t500x500")
             else:
                 raise InvalidSourceError(self.client.source)
@@ -151,7 +147,17 @@ class Track:
 
         self.folder = kwargs["parent_folder"] or self.folder
 
+        if not self.part_of_tracklist and kwargs["add_singles_to_folder"]:
+            self.folder = os.path.join(
+                self.folder,
+                clean_format(
+                    kwargs.get("folder_format", FOLDER_FORMAT),
+                    self.meta.get_album_formatter(self.quality),
+                ),
+            )
+
         self.file_format = kwargs.get("track_format", TRACK_FORMAT)
+
         self.folder = sanitize_filepath(self.folder, platform="auto")
         self.format_final_path()
 
@@ -175,9 +181,7 @@ class Track:
             return False
 
         self.download_cover()  # only downloads for playlists and singles
-        self.path = os.path.join(
-            gettempdir(), f"{hash(self.id)}_{self.quality}.tmp"
-        )
+        self.path = os.path.join(gettempdir(), f"{hash(self.id)}_{self.quality}.tmp")
         return True
 
     def download(
@@ -352,9 +356,7 @@ class Track:
         if not hasattr(self, "cover_url"):
             return False
 
-        self.cover_path = os.path.join(
-            gettempdir(), f"cover{hash(self.cover_url)}.jpg"
-        )
+        self.cover_path = os.path.join(gettempdir(), f"cover{hash(self.cover_url)}.jpg")
         logger.debug(f"Downloading cover from {self.cover_url}")
         # click.secho(f"\nDownloading cover art for {self!s}", fg="blue")
 
@@ -374,21 +376,19 @@ class Track:
         a dict with the keys allowed in formatter strings, and their values in
         the TrackMetadata object.
         """
-        formatter = self.meta.get_formatter()
+        formatter = self.meta.get_formatter(max_quality=self.quality)
         logger.debug("Track meta formatter %s", formatter)
         filename = clean_format(self.file_format, formatter)
-        self.final_path = os.path.join(self.folder, filename)[
-            :250
-        ].strip() + ext(self.quality, self.client.source)
+        self.final_path = os.path.join(self.folder, filename)[:250].strip() + ext(
+            self.quality, self.client.source
+        )
 
         logger.debug("Formatted path: %s", self.final_path)
 
         return self.final_path
 
     @classmethod
-    def from_album_meta(
-        cls, album: TrackMetadata, track: dict, client: Client
-    ):
+    def from_album_meta(cls, album: TrackMetadata, track: dict, client: Client):
         """Return a new Track object initialized with info.
 
         :param album: album metadata returned by API
@@ -398,7 +398,7 @@ class Track:
         :raises IndexError
         """
         meta = TrackMetadata(album=album, track=track, source=client.source)
-        return cls(client=client, meta=meta, id=track["id"])
+        return cls(client=client, meta=meta, id=track["id"], part_of_tracklist=True)
 
     @classmethod
     def from_api(cls, item: dict, client: Client):
@@ -456,9 +456,7 @@ class Track:
         :param embed_cover: Embed cover art into file
         :type embed_cover: bool
         """
-        assert isinstance(
-            self.meta, TrackMetadata
-        ), "meta must be TrackMetadata"
+        assert isinstance(self.meta, TrackMetadata), "meta must be TrackMetadata"
         if not self.downloaded:
             logger.info(
                 "Track %s not tagged because it was not downloaded",
@@ -554,9 +552,7 @@ class Track:
         """
         if not self.downloaded:
             logger.debug("Track not downloaded, skipping conversion")
-            click.secho(
-                "Track not downloaded, skipping conversion", fg="magenta"
-            )
+            click.secho("Track not downloaded, skipping conversion", fg="magenta")
             return
 
         CONV_CLASS = {
@@ -570,17 +566,18 @@ class Track:
             "M4A": converter.AAC,
         }
 
-        self.container = codec.upper()
+        try:
+            self.container = codec.upper()
+        except AttributeError:
+            click.secho("Error: No audio codec chosen to convert to.", fg="red")
+            raise click.Abort
+
         if not hasattr(self, "final_path"):
             self.format_final_path()
 
         if not os.path.isfile(self.path):
-            logger.info(
-                "File %s does not exist. Skipping conversion.", self.path
-            )
-            click.secho(
-                f"{self!s} does not exist. Skipping conversion.", fg="red"
-            )
+            logger.info("File %s does not exist. Skipping conversion.", self.path)
+            click.secho(f"{self!s} does not exist. Skipping conversion.", fg="red")
             return
 
         assert (
@@ -847,9 +844,7 @@ class Tracklist(list):
             # Tidal errors out with unlimited concurrency
             # max_workers = 15 if self.client.source == "tidal" else 90
             with concurrent.futures.ThreadPoolExecutor(15) as executor:
-                futures = [
-                    executor.submit(target, item, **kwargs) for item in self
-                ]
+                futures = [executor.submit(target, item, **kwargs) for item in self]
                 try:
                     concurrent.futures.wait(futures)
                 except (KeyboardInterrupt, SystemExit):
