@@ -2,6 +2,7 @@
 
 import logging
 import os
+import shutil
 from getpass import getpass
 from hashlib import md5
 
@@ -14,7 +15,6 @@ from .config import Config
 from .constants import CACHE_DIR, CONFIG_DIR, CONFIG_PATH, QOBUZ_FEATURED_KEYS
 from .core import MusicDL
 
-
 logging.basicConfig(level="WARNING")
 logger = logging.getLogger("streamrip")
 
@@ -25,9 +25,7 @@ if not os.path.isdir(CACHE_DIR):
 
 
 @click.group(invoke_without_command=True)
-@click.option(
-    "-c", "--convert", metavar="CODEC", help="alac, mp3, flac, or ogg"
-)
+@click.option("-c", "--convert", metavar="CODEC", help="alac, mp3, flac, or ogg")
 @click.option(
     "-u",
     "--urls",
@@ -69,7 +67,7 @@ def cli(ctx, **kwargs):
     if ctx.invoked_subcommand == "config":
         return
 
-    if config.session["check_for_updates"]:
+    if config.session["misc"]["check_for_updates"]:
         r = requests.get("https://pypi.org/pypi/streamrip/json").json()
         newest = r["info"]["version"]
         if __version__ != newest:
@@ -140,9 +138,7 @@ def filter_discography(ctx, **kwargs):
 
 
 @cli.command()
-@click.option(
-    "-t", "--type", default="album", help="album, playlist, track, or artist"
-)
+@click.option("-t", "--type", default="album", help="album, playlist, track, or artist")
 @click.option(
     "-s",
     "--source",
@@ -263,9 +259,7 @@ def lastfm(ctx, source, url):
 
 @cli.command()
 @click.option("-o", "--open", is_flag=True, help="Open the config file")
-@click.option(
-    "-d", "--directory", is_flag=True, help="Open the config directory"
-)
+@click.option("-d", "--directory", is_flag=True, help="Open the config directory")
 @click.option("-q", "--qobuz", is_flag=True, help="Set Qobuz credentials")
 @click.option("-t", "--tidal", is_flag=True, help="Re-login into Tidal")
 @click.option("--reset", is_flag=True, help="RESET the config file")
@@ -275,6 +269,12 @@ def lastfm(ctx, source, url):
     help="Reset the config file, keeping the credentials",
 )
 @click.option("-p", "--path", is_flag=True, help="Show the config file's path")
+@click.option(
+    "-ov",
+    "--open-vim",
+    is_flag=True,
+    help="Open the config file in the nvim or vim text editor.",
+)
 @click.pass_context
 def config(ctx, **kwargs):
     """Manage the streamrip configuration file."""
@@ -285,21 +285,25 @@ def config(ctx, **kwargs):
         config.update()
 
     if kwargs["path"]:
-        print(CONFIG_PATH)
+        click.echo(CONFIG_PATH)
 
     if kwargs["open"]:
         click.secho(f"Opening {CONFIG_PATH}", fg="green")
         click.launch(CONFIG_PATH)
 
+    if kwargs["open_vim"]:
+        if shutil.which("nvim") is not None:
+            os.system(f"nvim '{CONFIG_PATH}'")
+        else:
+            os.system(f"vim '{CONFIG_PATH}'")
+
     if kwargs["directory"]:
         config_dir = os.path.dirname(CONFIG_PATH)
         click.secho(f"Opening {config_dir}", fg="green")
         click.launch(config_dir)
 
     if kwargs["qobuz"]:
-        config.file["qobuz"]["email"] = input(
-            click.style("Qobuz email: ", fg="blue")
-        )
+        config.file["qobuz"]["email"] = input(click.style("Qobuz email: ", fg="blue"))
 
         click.secho("Qobuz password (will not show on screen):", fg="blue")
         config.file["qobuz"]["password"] = md5(
@@ -307,9 +311,7 @@ def config(ctx, **kwargs):
         ).hexdigest()
 
         config.save()
-        click.secho(
-            "Qobuz credentials hashed and saved to config.", fg="green"
-        )
+        click.secho("Qobuz credentials hashed and saved to config.", fg="green")
 
     if kwargs["tidal"]:
         client = TidalClient()
