@@ -10,7 +10,7 @@ APPNAME = "streamrip"
 
 CACHE_DIR = click.get_app_dir(APPNAME)
 CONFIG_DIR = click.get_app_dir(APPNAME)
-CONFIG_PATH = os.path.join(CONFIG_DIR, "config.yaml")
+CONFIG_PATH = os.path.join(CONFIG_DIR, "config.toml")
 LOG_DIR = click.get_app_dir(APPNAME)
 DB_PATH = os.path.join(LOG_DIR, "downloads.db")
 
@@ -19,9 +19,7 @@ DOWNLOADS_DIR = os.path.join(HOME, "StreamripDownloads")
 
 AGENT = "Mozilla/5.0 (Windows NT 10.0; Win64; x64; rv:83.0) Gecko/20100101 Firefox/83.0"
 
-TIDAL_COVER_URL = (
-    "https://resources.tidal.com/images/{uuid}/{width}x{height}.jpg"
-)
+TIDAL_COVER_URL = "https://resources.tidal.com/images/{uuid}/{width}x{height}.jpg"
 
 
 QUALITY_DESC = {
@@ -32,7 +30,6 @@ QUALITY_DESC = {
     4: "24bit/192kHz",
 }
 
-
 QOBUZ_FEATURED_KEYS = (
     "most-streamed",
     "recent-releases",
@@ -144,7 +141,9 @@ ALBUM_KEYS = (
     "composer",
 )
 # TODO: rename these to DEFAULT_FOLDER_FORMAT etc
-FOLDER_FORMAT = "{albumartist} - {title} ({year}) [{container}] [{bit_depth}B-{sampling_rate}kHz]"
+FOLDER_FORMAT = (
+    "{albumartist} - {title} ({year}) [{container}] [{bit_depth}B-{sampling_rate}kHz]"
+)
 TRACK_FORMAT = "{tracknumber}. {artist} - {title}"
 
 
@@ -159,6 +158,7 @@ LASTFM_URL_REGEX = r"https://www.last.fm/user/\w+/playlists/\w+"
 QOBUZ_INTERPRETER_URL_REGEX = (
     r"https?://www\.qobuz\.com/\w\w-\w\w/interpreter/[-\w]+/[-\w]+"
 )
+DEEZER_DYNAMIC_LINK_REGEX = r"https://deezer\.page\.link/\w+"
 YOUTUBE_URL_REGEX = r"https://www\.youtube\.com/watch\?v=[-\w]+"
 
 TIDAL_MAX_Q = 7
