@@ -5,7 +5,6 @@ import html
 import logging
 import os
 import re
-import sys
 from getpass import getpass
 from hashlib import md5
 from string import Formatter
@@ -33,16 +32,18 @@ from .constants import (
     SOUNDCLOUD_URL_REGEX,
     URL_REGEX,
     YOUTUBE_URL_REGEX,
+    DEEZER_DYNAMIC_LINK_REGEX,
 )
 from .db import MusicDB
 from .exceptions import (
     AuthenticationError,
+    MissingCredentials,
     NonStreamable,
     NoResultsFound,
     ParsingError,
 )
 from .tracklists import Album, Artist, Label, Playlist, Tracklist
-from .utils import extract_interpreter_url
+from .utils import extract_interpreter_url, extract_deezer_dynamic_link
 
 logger = logging.getLogger("streamrip")
 
@@ -82,6 +83,7 @@ class MusicDL(list):
         self.lastfm_url_parse = re.compile(LASTFM_URL_REGEX)
         self.interpreter_url_parse = re.compile(QOBUZ_INTERPRETER_URL_REGEX)
         self.youtube_url_parse = re.compile(YOUTUBE_URL_REGEX)
+        self.deezer_dynamic_url_parse = re.compile(DEEZER_DYNAMIC_LINK_REGEX)
 
         self.config: Config
         if config is None:
@@ -97,9 +99,11 @@ class MusicDL(list):
         }
 
         self.db: Union[MusicDB, list]
-        if self.config.session["database"]["enabled"]:
-            if self.config.session["database"]["path"] is not None:
-                self.db = MusicDB(self.config.session["database"]["path"])
+        db_settings = self.config.session["database"]
+        if db_settings["enabled"]:
+            path = db_settings["path"]
+            if path:
+                self.db = MusicDB(path)
             else:
                 self.db = MusicDB(DB_PATH)
                 self.config.file["database"]["path"] = DB_PATH
@@ -173,46 +177,42 @@ class MusicDL(list):
 
         :rtype: dict
         """
+        session = self.config.session
+        logger.debug(session)
+        # So that the dictionary isn't searched for the same keys multiple times
+        artwork, conversion, filepaths = tuple(
+            session[key] for key in ("artwork", "conversion", "filepaths")
+        )
         return {
             "database": self.db,
-            "parent_folder": self.config.session["downloads"]["folder"],
-            "folder_format": self.config.session["path_format"]["folder"],
-            "track_format": self.config.session["path_format"]["track"],
-            "embed_cover": self.config.session["artwork"]["embed"],
-            "embed_cover_size": self.config.session["artwork"]["size"],
-            "keep_hires_cover": self.config.session["artwork"][
-                "keep_hires_cover"
-            ],
-            "set_playlist_to_album": self.config.session["metadata"][
-                "set_playlist_to_album"
-            ],
-            "stay_temp": self.config.session["conversion"]["enabled"],
-            "conversion": self.config.session["conversion"],
-            "concurrent_downloads": self.config.session[
-                "concurrent_downloads"
-            ],
-            "new_tracknumbers": self.config.session["metadata"][
-                "new_playlist_tracknumbers"
-            ],
-            "download_videos": self.config.session["tidal"]["download_videos"],
-            "download_booklets": self.config.session["qobuz"][
-                "download_booklets"
-            ],
-            "download_youtube_videos": self.config.session["youtube"][
-                "download_videos"
-            ],
-            "youtube_video_downloads_folder": self.config.session["youtube"][
+            "parent_folder": session["downloads"]["folder"],
+            "folder_format": filepaths["folder_format"],
+            "track_format": filepaths["track_format"],
+            "embed_cover": session["artwork"]["embed"],
+            "embed_cover_size": artwork["size"],
+            "keep_hires_cover": artwork["keep_hires_cover"],
+            "set_playlist_to_album": session["metadata"]["set_playlist_to_album"],
+            "stay_temp": conversion["enabled"],
+            "conversion": conversion,
+            "concurrent_downloads": session["downloads"]["concurrent"],
+            "new_tracknumbers": session["metadata"]["new_playlist_tracknumbers"],
+            "download_videos": session["tidal"]["download_videos"],
+            "download_booklets": session["qobuz"]["download_booklets"],
+            "download_youtube_videos": session["youtube"]["download_videos"],
+            "youtube_video_downloads_folder": session["youtube"][
                 "video_downloads_folder"
             ],
+            "add_singles_to_folder": filepaths["add_singles_to_folder"],
         }
 
     def download(self):
         """Download all the items in self."""
         try:
             arguments = self._get_download_args()
-        except KeyError:
+        except KeyError as e:
             self._config_updating_message()
             self.config.update()
+            logger.debug("Config update error: %s", e)
             exit()
         except Exception as err:
             self._config_corrupted_message(err)
@@ -220,9 +220,7 @@ class MusicDL(list):
 
         logger.debug("Arguments from config: %s", arguments)
 
-        source_subdirs = self.config.session["downloads"][
-            "source_subdirectories"
-        ]
+        source_subdirs = self.config.session["downloads"]["source_subdirectories"]
         for item in self:
             if source_subdirs:
                 arguments["parent_folder"] = self.__get_source_subdir(
@@ -233,26 +231,20 @@ class MusicDL(list):
                 item.download(**arguments)
                 continue
 
-            arguments["quality"] = self.config.session[item.client.source][
-                "quality"
-            ]
+            arguments["quality"] = self.config.session[item.client.source]["quality"]
             if isinstance(item, Artist):
                 filters_ = tuple(
                     k for k, v in self.config.session["filters"].items() if v
                 )
                 arguments["filters"] = filters_
-                logger.debug(
-                    "Added filter argument for artist/label: %s", filters_
-                )
+                logger.debug("Added filter argument for artist/label: %s", filters_)
 
             if not (isinstance(item, Tracklist) and item.loaded):
                 logger.debug("Loading metadata")
                 try:
                     item.load_meta()
                 except NonStreamable:
-                    click.secho(
-                        f"{item!s} is not available, skipping.", fg="red"
-                    )
+                    click.secho(f"{item!s} is not available, skipping.", fg="red")
                     continue
 
             item.download(**arguments)
@@ -291,6 +283,12 @@ class MusicDL(list):
                 except AuthenticationError:
                     click.secho("Invalid credentials, try again.")
                     self.prompt_creds(client.source)
+                    creds = self.config.creds(client.source)
+                except MissingCredentials:
+                    logger.debug("Credentials are missing. Prompting..")
+                    self.prompt_creds(client.source)
+                    creds = self.config.creds(client.source)
+
             if (
                 client.source == "qobuz"
                 and not creds.get("secrets")
@@ -312,7 +310,6 @@ class MusicDL(list):
             https://www.qobuz.com/us-en/{type}/{name}/{id}
             https://open.qobuz.com/{type}/{id}
             https://play.qobuz.com/{type}/{id}
-            /us-en/{type}/-/{id}
 
             https://www.deezer.com/us/{type}/{id}
             https://tidal.com/browse/{type}/{id}
@@ -334,11 +331,21 @@ class MusicDL(list):
             )
             url = self.interpreter_url_parse.sub("", url)
 
+        dynamic_urls = self.deezer_dynamic_url_parse.findall(url)
+        if dynamic_urls:
+            click.secho(
+                "Extracting IDs from Deezer dynamic link. Use urls "
+                "of the form https://www.deezer.com/{country}/{type}/{id} for "
+                "faster processing.",
+                fg="yellow",
+            )
+            parsed.extend(
+                ("deezer", *extract_deezer_dynamic_link(url)) for url in dynamic_urls
+            )
+
         parsed.extend(self.url_parse.findall(url))  # Qobuz, Tidal, Dezer
         soundcloud_urls = self.soundcloud_url_parse.findall(url)
-        soundcloud_items = [
-            self.clients["soundcloud"].get(u) for u in soundcloud_urls
-        ]
+        soundcloud_items = [self.clients["soundcloud"].get(u) for u in soundcloud_urls]
 
         parsed.extend(
             ("soundcloud", item["kind"], url)
@@ -370,15 +377,11 @@ class MusicDL(list):
 
         # For testing:
         # https://www.last.fm/user/nathan3895/playlists/12058911
-        user_regex = re.compile(
-            r"https://www\.last\.fm/user/([^/]+)/playlists/\d+"
-        )
+        user_regex = re.compile(r"https://www\.last\.fm/user/([^/]+)/playlists/\d+")
         lastfm_urls = self.lastfm_url_parse.findall(urls)
         try:
             lastfm_source = self.config.session["lastfm"]["source"]
-            lastfm_fallback_source = self.config.session["lastfm"][
-                "fallback_source"
-            ]
+            lastfm_fallback_source = self.config.session["lastfm"]["fallback_source"]
         except KeyError:
             self._config_updating_message()
             self.config.update()
@@ -408,9 +411,7 @@ class MusicDL(list):
                 except (NoResultsFound, StopIteration):
                     return None
 
-            track = try_search(lastfm_source) or try_search(
-                lastfm_fallback_source
-            )
+            track = try_search(lastfm_source) or try_search(lastfm_fallback_source)
             if track is None:
                 return False
 
@@ -432,9 +433,7 @@ class MusicDL(list):
                 pl.creator = creator_match.group(1)
 
             tracks_not_found = 0
-            with concurrent.futures.ThreadPoolExecutor(
-                max_workers=15
-            ) as executor:
+            with concurrent.futures.ThreadPoolExecutor(max_workers=15) as executor:
                 futures = [
                     executor.submit(search_query, title, artist, pl)
                     for title, artist in queries
@@ -451,9 +450,7 @@ class MusicDL(list):
             pl.loaded = True
 
             if tracks_not_found > 0:
-                click.secho(
-                    f"{tracks_not_found} tracks not found.", fg="yellow"
-                )
+                click.secho(f"{tracks_not_found} tracks not found.", fg="yellow")
             self.append(pl)
 
     def handle_txt(self, filepath: Union[str, os.PathLike]):
@@ -508,9 +505,7 @@ class MusicDL(list):
         else:
             logger.debug("Not generator")
             items = (
-                results.get("data")
-                or results.get("items")
-                or results.get("collection")
+                results.get("data") or results.get("items") or results.get("collection")
             )
             if items is None:
                 raise NoResultsFound(query)
@@ -550,9 +545,7 @@ class MusicDL(list):
             raise NotImplementedError
 
         fields = (fname for _, fname, _, _ in Formatter().parse(fmt) if fname)
-        ret = fmt.format(
-            **{k: media.get(k, default="Unknown") for k in fields}
-        )
+        ret = fmt.format(**{k: media.get(k, default="Unknown") for k in fields})
         return ret
 
     def interactive_search(  # noqa
@@ -574,6 +567,8 @@ class MusicDL(list):
                 return f"{res[0]+1}. {res[1].album}"
             elif isinstance(res[1], Track):
                 return f"{res[0]+1}. {res[1].meta.title}"
+            elif isinstance(res[1], Playlist):
+                return f"{res[0]+1}. {res[1].name}"
             else:
                 raise NotImplementedError(type(res[1]).__name__)
 
@@ -587,14 +582,7 @@ class MusicDL(list):
             return self.preview_media(results[int("".join(num)) - 1])
 
         if os.name == "nt":
-            try:
-                from pick import pick
-            except (ImportError, ModuleNotFoundError):
-                click.secho(
-                    "Run `pip3 install windows-curses` to use interactive mode.",
-                    fg="red",
-                )
-                sys.exit()
+            from pick import pick
 
             choice = pick(
                 tuple(enumerate(results)),
@@ -614,14 +602,7 @@ class MusicDL(list):
 
             return True
         else:
-            try:
-                from simple_term_menu import TerminalMenu
-            except (ImportError, ModuleNotFoundError):
-                click.secho(
-                    "Run `pip3 install simple-term-menu` to use interactive mode.",
-                    fg="red",
-                )
-                sys.exit()
+            from simple_term_menu import TerminalMenu
 
             menu = TerminalMenu(
                 map(title, enumerate(results)),
@@ -665,6 +646,7 @@ class MusicDL(list):
 
         def essence(s):
             s = re.sub(r"&#\d+;", "", s)  # remove HTML entities
+            # TODO: change to finditer
             return "".join(words.findall(s))
 
         def get_titles(s):
@@ -694,9 +676,7 @@ class MusicDL(list):
         playlist_title = html.unescape(playlist_title_match.group(1))
 
         if remaining_tracks > 0:
-            with concurrent.futures.ThreadPoolExecutor(
-                max_workers=15
-            ) as executor:
+            with concurrent.futures.ThreadPoolExecutor(max_workers=15) as executor:
                 last_page = int(remaining_tracks // 50) + int(
                     remaining_tracks % 50 != 0
                 )
@@ -738,7 +718,8 @@ class MusicDL(list):
 
             self.config.save()
             click.secho(
-                f'Credentials saved to config file at "{self.config._path}"'
+                f'Credentials saved to config file at "{self.config._path}"',
+                fg="green",
             )
         else:
             raise Exception
