@@ -8,6 +8,7 @@ from collections import OrderedDict
 from typing import Generator, Hashable, Iterable, Optional, Union
 
 from .constants import (
+    ALBUM_KEYS,
     COPYRIGHT,
     COVER_SIZES,
     FLAC_KEY,
@@ -130,9 +131,7 @@ class TrackMetadata:
             self.album = resp.get("title", "Unknown Album")
             self.tracktotal = resp.get("tracks_count", 1)
             self.genre = resp.get("genres_list") or resp.get("genre")
-            self.date = resp.get("release_date_original") or resp.get(
-                "release_date"
-            )
+            self.date = resp.get("release_date_original") or resp.get("release_date")
             self.copyright = resp.get("copyright")
             self.albumartist = safe_get(resp, "artist", "name")
             self.composer = safe_get(resp, "composer", "name")
@@ -141,9 +140,7 @@ class TrackMetadata:
             self.disctotal = (
                 max(
                     track.get("media_number", 1)
-                    for track in safe_get(
-                        resp, "tracks", "items", default=[{}]
-                    )
+                    for track in safe_get(resp, "tracks", "items", default=[{}])
                 )
                 or 1
             )
@@ -155,9 +152,7 @@ class TrackMetadata:
             # Non-embedded information
             self.version = resp.get("version")
             self.cover_urls = OrderedDict(resp["image"])
-            self.cover_urls["original"] = self.cover_urls["large"].replace(
-                "600", "org"
-            )
+            self.cover_urls["original"] = self.cover_urls["large"].replace("600", "org")
             self.streamable = resp.get("streamable", False)
             self.bit_depth = resp.get("maximum_bit_depth")
             self.sampling_rate = resp.get("maximum_sampling_rate")
@@ -191,22 +186,14 @@ class TrackMetadata:
             )
             self.streamable = resp.get("allowStreaming", False)
 
-            if q := resp.get(
-                "audioQuality"
-            ):  # for album entries in single tracks
+            if q := resp.get("audioQuality"):  # for album entries in single tracks
                 self._get_tidal_quality(q)
 
         elif self.__source == "deezer":
             self.album = resp.get("title", "Unknown Album")
-            self.tracktotal = resp.get("track_total", 0) or resp.get(
-                "nb_tracks", 0
-            )
+            self.tracktotal = resp.get("track_total", 0) or resp.get("nb_tracks", 0)
             self.disctotal = (
-                max(
-                    track.get("disk_number")
-                    for track in resp.get("tracks", [{}])
-                )
-                or 1
+                max(track.get("disk_number") for track in resp.get("tracks", [{}])) or 1
             )
             self.genre = safe_get(resp, "genres", "data")
             self.date = resp.get("release_date")
@@ -370,9 +357,7 @@ class TrackMetadata:
 
         if isinstance(self._genres, list):
             if self.__source == "qobuz":
-                genres: Iterable = re.findall(
-                    r"([^\u2192\/]+)", "/".join(self._genres)
-                )
+                genres: Iterable = re.findall(r"([^\u2192\/]+)", "/".join(self._genres))
                 genres = set(genres)
             elif self.__source == "deezer":
                 genres = ", ".join(g["name"] for g in self._genres)
@@ -404,9 +389,7 @@ class TrackMetadata:
         if hasattr(self, "_copyright"):
             if self._copyright is None:
                 return None
-            copyright: str = re.sub(
-                r"(?i)\(P\)", PHON_COPYRIGHT, self._copyright
-            )
+            copyright: str = re.sub(r"(?i)\(P\)", PHON_COPYRIGHT, self._copyright)
             copyright = re.sub(r"(?i)\(C\)", COPYRIGHT, copyright)
             return copyright
 
@@ -446,7 +429,7 @@ class TrackMetadata:
         """
         self._year = val
 
-    def get_formatter(self) -> dict:
+    def get_formatter(self, max_quality: int) -> dict:
         """Return a dict that is used to apply values to file format strings.
 
         :rtype: dict
@@ -454,6 +437,18 @@ class TrackMetadata:
         # the keys in the tuple are the possible keys for format strings
         return {k: getattr(self, k) for k in TRACK_KEYS}
 
+    def get_album_formatter(self, max_quality: int) -> dict:
+        """Return a dict that is used to apply values to file format strings.
+
+        :param max_quality:
+        :type max_quality: int
+        :rtype: dict
+        """
+        formatter = {k: self.get(k) for k in ALBUM_KEYS}
+        formatter["container"] = "FLAC" if max_quality >= 2 else "MP3"
+        formatter["sampling_rate"] /= 1000
+        return formatter
+
     def tags(self, container: str = "flac") -> Generator:
         """Create a generator of key, value pairs for use with mutagen.
 
@@ -598,9 +593,7 @@ class TrackMetadata:
 
         :rtype: int
         """
-        return sum(
-            hash(v) for v in self.asdict().values() if isinstance(v, Hashable)
-        )
+        return sum(hash(v) for v in self.asdict().values() if isinstance(v, Hashable))
 
     def __repr__(self) -> str:
         """Return the string representation of the metadata object.
