@@ -20,6 +20,7 @@ from .metadata import TrackMetadata
 from .utils import (
     clean_format,
     get_container,
+    get_stats_from_quality,
     safe_get,
     tidal_cover_url,
     tqdm_download,
@@ -104,10 +105,7 @@ class Album(Tracklist):
         """
         # Generate the folder name
         self.folder_format = kwargs.get("folder_format", FOLDER_FORMAT)
-        if not hasattr(self, "quality"):
-            self.quality = min(
-                kwargs.get("quality", 3), self.client.max_quality
-            )
+        self.quality = min(kwargs.get("quality", 3), self.client.max_quality)
 
         self.folder = self._get_formatted_folder(
             kwargs.get("parent_folder", "StreamripDownloads"), self.quality
@@ -133,9 +131,7 @@ class Album(Tracklist):
             tqdm_download(cover_url, cover_path)
 
         hires_cov_path = os.path.join(self.folder, "cover.jpg")
-        if kwargs.get("keep_hires_cover", True) and not os.path.exists(
-            hires_cov_path
-        ):
+        if kwargs.get("keep_hires_cover", True) and not os.path.exists(hires_cov_path):
             tqdm_download(self.cover_urls["original"], hires_cov_path)
 
         cover_size = os.path.getsize(cover_path)
@@ -186,9 +182,7 @@ class Album(Tracklist):
         """
         logger.debug("Downloading track to %s", self.folder)
         if self.disctotal > 1 and isinstance(track, Track):
-            disc_folder = os.path.join(
-                self.folder, f"Disc {track.meta.discnumber}"
-            )
+            disc_folder = os.path.join(self.folder, f"Disc {track.meta.discnumber}")
             kwargs["parent_folder"] = disc_folder
         else:
             kwargs["parent_folder"] = self.folder
@@ -238,20 +232,25 @@ class Album(Tracklist):
                 )
 
     def _get_formatter(self) -> dict:
-        """Get a formatter that is used for previews in core.py.
+        """Get a formatter that is used for naming folders and previews.
 
         :rtype: dict
         """
-        fmt = dict()
-        for key in ALBUM_KEYS:
-            # default to None
-            fmt[key] = self.get(key)
+        fmt = {key: self.get(key) for key in ALBUM_KEYS}
 
-        if fmt.get("sampling_rate", False):
-            fmt["sampling_rate"] /= 1000
-            # change 48.0kHz -> 48kHz, 44.1kHz -> 44.1kHz
-            if fmt["sampling_rate"] % 1 == 0.0:
-                fmt["sampling_rate"] = int(fmt["sampling_rate"])
+        stats = get_stats_from_quality(self.quality)
+
+        # The quality chosen is not the maximum available quality
+        if stats != (fmt.get("sampling_rate"), fmt.get("bit_depth")):
+            fmt["bit_depth"] = stats[0]
+            fmt["sampling_rate"] = stats[1]
+
+        if sr := fmt.get("sampling_rate"):
+            if sr % 1000 == 0:
+                # truncate the decimal .0 when converting to str
+                fmt["sampling_rate"] = int(sr / 1000)
+            else:
+                fmt["sampling_rate"] = sr / 1000
 
         return fmt
 
@@ -270,9 +269,7 @@ class Album(Tracklist):
             # lossy codecs don't have these metrics
             self.bit_depth = self.sampling_rate = None
 
-        formatted_folder = clean_format(
-            self.folder_format, self._get_formatter()
-        )
+        formatted_folder = clean_format(self.folder_format, self._get_formatter())
 
         return os.path.join(parent_folder, formatted_folder)
 
@@ -385,9 +382,7 @@ class Playlist(Tracklist):
         if self.client.source == "qobuz":
             self.name = self.meta["name"]
             self.image = self.meta["images"]
-            self.creator = safe_get(
-                self.meta, "owner", "name", default="Qobuz"
-            )
+            self.creator = safe_get(self.meta, "owner", "name", default="Qobuz")
 
             tracklist = self.meta["tracks"]["items"]
 
@@ -400,9 +395,7 @@ class Playlist(Tracklist):
         elif self.client.source == "tidal":
             self.name = self.meta["title"]
             self.image = tidal_cover_url(self.meta["image"], 640)
-            self.creator = safe_get(
-                self.meta, "creator", "name", default="TIDAL"
-            )
+            self.creator = safe_get(self.meta, "creator", "name", default="TIDAL")
 
             tracklist = self.meta["tracks"]
 
@@ -419,9 +412,7 @@ class Playlist(Tracklist):
         elif self.client.source == "deezer":
             self.name = self.meta["title"]
             self.image = self.meta["picture_big"]
-            self.creator = safe_get(
-                self.meta, "creator", "name", default="Deezer"
-            )
+            self.creator = safe_get(self.meta, "creator", "name", default="Deezer")
 
             tracklist = self.meta["tracks"]
 
@@ -459,18 +450,19 @@ class Playlist(Tracklist):
                         id=track.get("id"),
                         meta=meta,
                         cover_url=gen_cover(track),
+                        part_of_tracklist=True,
                     )
                 )
 
         logger.debug(f"Loaded {len(self)} tracks from playlist {self.name}")
 
-    def _prepare_download(
-        self, parent_folder: str = "StreamripDownloads", **kwargs
-    ):
+    def _prepare_download(self, parent_folder: str = "StreamripDownloads", **kwargs):
         fname = sanitize_filename(self.name)
         self.folder = os.path.join(parent_folder, fname)
 
-        self.__download_index = 1  # used for tracknumbers
+        # Used for safe concurrency with tracknumbers instead of an object
+        # level that stores an index
+        self.__indices = iter(range(1, len(self) + 1))
         self.download_message()
 
     def _download_item(self, item: Track, **kwargs) -> bool:  # type: ignore
@@ -479,20 +471,30 @@ class Playlist(Tracklist):
             item.load_meta()
             click.secho(f"Downloading {item!s}", fg="blue")
 
-        if kwargs.get("set_playlist_to_album", False):
+        if playlist_to_album := kwargs.get("set_playlist_to_album", False):
             item["album"] = self.name
             item["albumartist"] = self.creator
 
         if kwargs.get("new_tracknumbers", True):
-            item["tracknumber"] = self.__download_index
+            item["tracknumber"] = next(self.__indices)
             item["discnumber"] = 1
-            self.__download_index += 1
 
         self.downloaded = item.download(**kwargs)
 
         if self.downloaded and self.client.source != "deezer":
             item.tag(embed_cover=kwargs.get("embed_cover", True))
 
+        if playlist_to_album and self.client.source == "deezer":
+            # Because Deezer tracks come pre-tagged, the `set_playlist_to_album`
+            # option is never set. Here, we manually do this
+            from mutagen.flac import FLAC
+
+            audio = FLAC(item.path)
+            audio["ALBUM"] = self.name
+            audio["ALBUMARTIST"] = self.creator
+            audio["TRACKNUMBER"] = f"{item['tracknumber']:02}"
+            audio.save()
+
         return self.downloaded
 
     @staticmethod
@@ -612,7 +614,7 @@ class Artist(Tracklist):
             albums = self.meta["albums"]
 
         elif self.client.source == "deezer":
-            # TODO: load artist name
+            self.name = self.meta["name"]
             albums = self.meta["albums"]
 
         else:
@@ -652,9 +654,7 @@ class Artist(Tracklist):
             final = self
 
         if isinstance(filters, tuple) and self.client.source == "qobuz":
-            filter_funcs = (
-                getattr(self, f"_{filter_}") for filter_ in filters
-            )
+            filter_funcs = (getattr(self, f"_{filter_}") for filter_ in filters)
             for func in filter_funcs:
                 final = filter(func, final)
 
@@ -773,10 +773,7 @@ class Artist(Tracklist):
             best_bd = bit_depth(a["bit_depth"] for a in group)
             best_sr = sampling_rate(a["sampling_rate"] for a in group)
             for album in group:
-                if (
-                    album["bit_depth"] == best_bd
-                    and album["sampling_rate"] == best_sr
-                ):
+                if album["bit_depth"] == best_bd and album["sampling_rate"] == best_sr:
                     yield album
                     break
 
