@@ -1,11 +1,13 @@
 """Miscellaneous utility functions."""
 
+from __future__ import annotations
+
 import base64
 import logging
 import os
 import re
 from string import Formatter
-from typing import Dict, Hashable, Optional, Union
+from typing import Dict, Hashable, Optional, Tuple, Union
 
 import click
 import requests
@@ -14,7 +16,7 @@ from requests.packages import urllib3
 from tqdm import tqdm
 
 from .constants import AGENT, TIDAL_COVER_URL
-from .exceptions import InvalidSourceError, NonStreamable
+from .exceptions import InvalidQuality, InvalidSourceError, NonStreamable
 
 urllib3.disable_warnings()
 logger = logging.getLogger("streamrip")
@@ -81,9 +83,7 @@ def get_quality(quality_id: int, source: str) -> Union[str, int]:
         raise InvalidSourceError(source)
 
     possible_keys = set(q_map.keys())
-    assert (
-        quality_id in possible_keys
-    ), f"{quality_id} must be in {possible_keys}"
+    assert quality_id in possible_keys, f"{quality_id} must be in {possible_keys}"
     return q_map[quality_id]
 
 
@@ -109,9 +109,22 @@ def get_quality_id(bit_depth: Optional[int], sampling_rate: Optional[int]):
         return 4
 
 
-def tqdm_download(
-    url: str, filepath: str, params: dict = None, desc: str = None
-):
+def get_stats_from_quality(
+    quality_id: int,
+) -> Tuple[Optional[int], Optional[int]]:
+    if quality_id <= 1:
+        return (None, None)
+    elif quality_id == 2:
+        return (16, 44100)
+    elif quality_id == 3:
+        return (24, 96000)
+    elif quality_id == 4:
+        return (24, 192000)
+    else:
+        raise InvalidQuality(quality_id)
+
+
+def tqdm_download(url: str, filepath: str, params: dict = None, desc: str = None):
     """Download a file with a progress bar.
 
     :param url: url to direct download
@@ -183,9 +196,7 @@ def tidal_cover_url(uuid, size):
     possibles = (80, 160, 320, 640, 1280)
     assert size in possibles, f"size must be in {possibles}"
 
-    return TIDAL_COVER_URL.format(
-        uuid=uuid.replace("-", "/"), height=size, width=size
-    )
+    return TIDAL_COVER_URL.format(uuid=uuid.replace("-", "/"), height=size, width=size)
 
 
 def init_log(path: Optional[str] = None, level: str = "DEBUG"):
@@ -287,9 +298,7 @@ def gen_threadsafe_session(
         headers = {}
 
     session = requests.Session()
-    adapter = requests.adapters.HTTPAdapter(
-        pool_connections=100, pool_maxsize=100
-    )
+    adapter = requests.adapters.HTTPAdapter(pool_connections=100, pool_maxsize=100)
     session.mount("https://", adapter)
     session.headers.update(headers)
     return session
@@ -328,6 +337,19 @@ def extract_interpreter_url(url: str) -> str:
     )
 
 
+deezer_id_link_regex = re.compile(r"https://www\.deezer\.com/[a-z]{2}/(album|artist|playlist|track)/(\d+)")
+
+
+def extract_deezer_dynamic_link(url: str) -> Tuple[str, str]:
+    session = gen_threadsafe_session({"User-Agent": AGENT})
+    r = session.get(url)
+    match = deezer_id_link_regex.search(r.text)
+    if match:
+        return match.group(1), match.group(2)
+
+    raise Exception("Unable to extract Deezer dynamic link.")
+
+
 def get_container(quality: int, source: str) -> str:
     """Get the file container given the quality.
 
@@ -346,3 +368,24 @@ def get_container(quality: int, source: str) -> str:
         return "AAC"
 
     return "MP3"
+
+
+class Version:
+    def __init__(self, version: str):
+        self.value = tuple(map(int, version.split(".")))
+
+    def __gt__(self, other) -> bool:
+        assert isinstance(other, Version)
+        for v1, v2 in zip(self.value, other.value):
+            if v1 > v2:
+                return True
+            elif v1 < v2:
+                return False
+        return False
+
+    def __lt__(self, other) -> bool:
+        return not self.__gt__(other) and not self.__eq__(other)
+
+    def __eq__(self, other) -> bool:
+        assert isinstance(other, Version)
+        return all(v1 == v2 for v1, v2 in zip(self.value, other.value))
